/**
 * Grammar of ALLL tupple used to manipulate with Belief Base.
 * License: BUT OPEN SOURCE LICENCE v.1
 * 
 * @author Jan Horacek <ihoracek@fit.vutbr.cz>
 */
grammar BBGrammar;

@header {
    package gdefalll.Grammars;
    import java.util.ArrayList;
}

@lexer::header{
	package gdefalll.Grammars;
	import java.util.ArrayList;
}

@members {
    private int error_starting_position = -1;
    private ArrayList<String> errors = new ArrayList<String>();
    public void displayRecognitionError(String[] tokenNames,
                                        RecognitionException e)
    {
        String hdr = getErrorHeader(e);
        String msg = getErrorMessage(e, tokenNames);
        errors.add(hdr + " " + msg);
        if (error_starting_position < 0)
        {
	  error_starting_position = e.charPositionInLine;
        }
    }
    public int getErrorStartingPosition()
    {
      return error_starting_position;
    }
    public ArrayList<String> getErrors()
    {
        return errors;
    }
}

parse
    :    tuple EOF
    ;

// tupple

tuple
	:	LEFT_PAREN value (COMMA value)* RIGHT_PAREN
	;
	
value
	:	WORD
	|	REGISTER
	|	NUMBER
	|	UNIFICATOR
	|	tuple
	;
	
// Grouping
LEFT_PAREN
	:	'('
	;
RIGHT_PAREN
	:	')'
	;

// Words, which include our operators
WORD
	:	('a'..'z' | 'A'..'Z') ('a'..'z' | 'A'..'Z' | '0'..'9' | '_')*
	;

// comma
COMMA
	:	','
	;

// Register substitution
REGISTER
	:	('&1' | '&2' | '&3')
	;

UNIFICATOR
	:	'_'
	;

/* A number: can be an integer value, or a decimal value */
NUMBER
    :    ('0'..'9')+
    ;

/* We're going to ignore all white space characters */
WS  
    :   (' ' | '\t' | '\r'| '\n')
    ;