/**
 * Grammar of ALLL plan.
 * License: BUT OPEN SOURCE LICENCE v.1
 * 
 * @author Jan Horacek <ihoracek@fit.vutbr.cz>
 */
grammar PlanGrammar;

@header {
    package gdefalll.Grammars;
    import java.util.ArrayList;
    import gdefalll.gui.actions.*;
}

@lexer::header{
	package gdefalll.Grammars;
	import java.util.ArrayList;
}

@members {
    private int error_starting_position = -1;
    private ArrayList<String> errors = new ArrayList<String>();
    private ArrayList<ALLLAction> actionList = new ArrayList<ALLLAction>();
    
    public ArrayList<ALLLAction> getActionList()
    {
    	return actionList;
    }
    
    public void displayRecognitionError(String[] tokenNames,
                                        RecognitionException e)
    {
        String hdr = getErrorHeader(e);
        String msg = getErrorMessage(e, tokenNames);
        errors.add(hdr + " " + msg);
        if (error_starting_position < 0)
        {
	  error_starting_position = e.charPositionInLine;
        }
    }
    public int getErrorStartingPosition()
    {
      return error_starting_position;
    }
    public ArrayList<String> getErrors()
    {
        return errors;
    }
}

parse
    :    (action)* EOF
    ;

// tupple

action
	:	platform_action
	|	builtin_action
	;

builtin_action
	:	'+' t1=tuple {	ALLLAction action = new AddBB();
						action.setParameters($t1.text);
						actionList.add(action);
						}
	|	'-' t2=tuple {	ALLLAction action = new DelBB();
						action.setParameters($t2.text);
						actionList.add(action);
						}
	|	'*' t3=tuple {	ALLLAction action = new CheckBB();
						action.setParameters($t3.text);
						actionList.add(action);
						}
	|	'?' tuple {	ALLLAction action = new CheckInputBase();
						actionList.add(action);
						}
	|	'!' LEFT_PAREN address5_1=NUMBER COMMA t5_1=tuple RIGHT_PAREN {	ALLLAction action = new MessageOutput();
																		action.setParameters($address5_1.text + ',' + $t5_1.text);
																		actionList.add(action);
																		}
	|	'!' LEFT_PAREN address5_2=NUMBER COMMA r5_2=REGISTER RIGHT_PAREN {	ALLLAction action = new MessageOutput();
																			action.setParameters($address5_2.text + ',' + $r5_2.text);
																			actionList.add(action);
																			}
	|	'^' LEFT_PAREN w6=WORD RIGHT_PAREN {	ALLLAction action = new RemotePlan();
												action.setParameters($w6.text);
												actionList.add(action);
												}
	|	'#' {	ALLLAction action = new CatchSection();
				actionList.add(action);
				}
	
	|	'@(' {InternalPlanStart actionStart = new InternalPlanStart(); actionList.add(actionStart);}
		(action)* ')'
							{	InternalPlanEnd actionStop = new InternalPlanEnd();
								
								actionStop.setNestedAction((ALLLAction)actionStart);
								actionStart.setNestedAction((ALLLAction)actionStop);								
								actionList.add(actionStop);
								}
//	|	'&' t7=tuple {System.out.println($t7.text);}
	;

platform_action
	:	PLATFORM_ACTION_IDENTIFIER platform_action_type
	;

platform_action_type
	:	LEFT_PAREN 'w' COMMA LEFT_PAREN pn1=NUMBER RIGHT_PAREN RIGHT_PAREN {	ALLLAction action = new WaitTime();
																				action.setParameters($pn1.text);
																				actionList.add(action);
																				}
	|	LEFT_PAREN 'w' COMMA pr1=REGISTER RIGHT_PAREN {	ALLLAction action = new WaitTime();
														action.setParameters($pr1.text);
														actionList.add(action);
														}
	|	LEFT_PAREN 's' RIGHT_PAREN {	ALLLAction action = new WaitMsg();
										actionList.add(action);
										}
	|	LEFT_PAREN 'a' RIGHT_PAREN {	ALLLAction action = new StartMsgWait();
										actionList.add(action);
										}
	|	LEFT_PAREN 'l' COMMA params_led=led_action_possible_params RIGHT_PAREN {}																				
	;

led_action_possible_params
	:	plr1=REGISTER {	ALLLAction action = new Leds();
						action.setParameters($plr1.text);
						actionList.add(action);
						}
	|	LEFT_PAREN 'r' RIGHT_PAREN {	ALLLAction action = new Leds();
										action.setParameters("RED TOGGLE");
										actionList.add(action);
										}
	|	LEFT_PAREN 'g' RIGHT_PAREN {	ALLLAction action = new Leds();
										action.setParameters("GREEN TOGGLE");
										actionList.add(action);
										}
	|	LEFT_PAREN 'y' RIGHT_PAREN {	ALLLAction action = new Leds();
										action.setParameters("YELLOW TOGGLE");
										actionList.add(action);
										}
	|	LEFT_PAREN 'r,1' RIGHT_PAREN {	ALLLAction action = new Leds();
										action.setParameters("RED ON");
										actionList.add(action);
										}
	|	LEFT_PAREN 'g,1' RIGHT_PAREN {	ALLLAction action = new Leds();
										action.setParameters("GREEN ON");
										actionList.add(action);
										}
	|	LEFT_PAREN 'y,1' RIGHT_PAREN {	ALLLAction action = new Leds();
										action.setParameters("YELLOW ON");
										actionList.add(action);
										}
	|	LEFT_PAREN 'r,0' RIGHT_PAREN {	ALLLAction action = new Leds();
										action.setParameters("RED OFF");
										actionList.add(action);
										}
	|	LEFT_PAREN 'g,0' RIGHT_PAREN {	ALLLAction action = new Leds();
										action.setParameters("GREEN OFF");
										actionList.add(action);
										}
	|	LEFT_PAREN 'y,0' RIGHT_PAREN {	ALLLAction action = new Leds();
										action.setParameters("YELLOW OFF");
										actionList.add(action);
										}
	;


tuple
	:	LEFT_PAREN value (COMMA value)* RIGHT_PAREN
	;
	
value
	:	WORD
	|	REGISTER
	|	NUMBER
	|	UNIFICATOR
	|	tuple
	;

PLATFORM_ACTION_IDENTIFIER
	:	'$'
	;
	
// Grouping
LEFT_PAREN
	:	'('
	;
RIGHT_PAREN
	:	')'
	;

// Words, which include our operators
WORD
	:	('a'..'z' | 'A'..'Z') ('a'..'z' | 'A'..'Z' | '0'..'9' | '_')*
	;

// comma
COMMA
	:	','
	;

// Register substitution
REGISTER
	:	('&1' | '&2' | '&3')
	;

UNIFICATOR
	:	'_'
	;

/* A number: can be an integer value, or a decimal value */
NUMBER
    :    ('0'..'9')+
    ;

/* We're going to ignore all white space characters */
WS  
    :   (' ' | '\t' | '\r'| '\n')
    ;