package gdefalll.gui;

import javax.swing.JPanel;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import gdefalll.gui.GuiWSNNode;
import gdefalll.events.StatusChangeGenerator;

/**
 * Topology class. It will be used in extended version with simulator.
 * License: BUT OPEN SOURCE LICENCE v.1
 * 
 * @author Jan Horacek <ihoracek@fit.vutbr.cz>
 */
public class GuiTopology extends JPanel implements MouseListener, MouseMotionListener, ComponentListener {

	private static final long serialVersionUID = 0L;

	public StatusChangeGenerator status = new StatusChangeGenerator();
	
	private ArrayList<GuiWSNNode> nodeList = new ArrayList<GuiWSNNode>();
	private GuiWSNNode nodeDragged = null;
	private static final Dimension DefaultSize = new Dimension(4000,3000); 
	JScrollPane scrollPane = new JScrollPane(this);
	private Color lineCBright = new Color(240, 240, 240);
	private Color lineCDark = new Color(208, 208, 208);
	
	/**
	 * Create the panel.
	 */
	public GuiTopology() {
		setPreferredSize(DefaultSize);
		addMouseListener(this);
		addMouseMotionListener(this);
		scrollPane.getHorizontalScrollBar().setValue(1); // has to be set virtually for the first time
		scrollPane.getVerticalScrollBar().setValue(1200);
		scrollPane.getHorizontalScrollBar().setValue(1400);
	}

	@Override
	public void paintComponent(Graphics g){
		/* Parent paintComponent method */
		super.paintComponent(g);
		/* Draw Background */
		drawBackground(g);
		drawNodes(g);
	}
	
	private void drawBackground(Graphics g) {
		g.setColor(Color.white);
		g.fillRect(0, 0, getWidth(), getHeight());
		
		Dimension d = getSize();
		int width = (int) d.getWidth();
		int height = (int) d.getHeight();
		
		/* Vertical lines */
		for(int i=0; i <= width; i+=10){
			Color c = (i%50==0) ? lineCDark : lineCBright;
			g.setColor(c);
			g.drawLine(i, 0, i, height);
		}
		/* Horizontal lines */
		for(int i=0; i <= height; i+=10){
			Color c = (i%50==0) ? lineCDark : lineCBright;
			g.setColor(c);
			g.drawLine(0, i, width, i);
		}
	}
	
	private void drawNodes(Graphics g) {
		Iterator<GuiWSNNode> ni = nodeList.iterator();
		while(ni.hasNext()){
			ni.next().paintComponent(g);
		}
	}
	
	private GuiWSNNode nodeAtPos(int x, int y){
		Iterator<GuiWSNNode> ni = nodeList.iterator();
		while(ni.hasNext()){
			GuiWSNNode node = ni.next();
			if (node.isAtPos(x, y))
			{
				return node;
			}
		}
		return null;
	}
	
	
	
	/////////////////////////////////////////////////////////////////////////////////
	//
	/////////////////////////////////////////////////////////////////////////////////
	@Override
	public void mouseClicked(MouseEvent e) {
		if (e.getClickCount() == 2)
		{
			
		}
		else if (e.getClickCount() == 1)
		{
			if (SwingUtilities.isLeftMouseButton(e))
			{
				if (nodeAtPos(e.getX(), e.getY()) != null) {
					// TODO Edit options
				} else {
					addEmptyNode(e);
				}
			}
			else
			{
				delNode(nodeAtPos(e.getX(), e.getY()));
			}
		}
		repaint();
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// Auto-generated method stub
		
	}

	@Override
	public void mouseExited(MouseEvent e) {
		// Auto-generated method stub
	}

	private int press_x, press_y;
	@Override
	public void mousePressed(MouseEvent e) {
		press_x = e.getX();
		press_y = e.getY();
		nodeDragged = nodeAtPos(e.getX(), e.getY());
		repaint();
	}
	
	@Override
	public void mouseReleased(MouseEvent e) {
		if (nodeDragged == null)
		{
			
		}
		nodeDragged = null;	
		repaint();
	}
	
	@Override
	public void mouseDragged(MouseEvent e) {
		if (nodeDragged != null){
			int dx = e.getX() - nodeDragged.getX();
			int dy = e.getY() - nodeDragged.getY();
			nodeDragged.moveX(dx);
			nodeDragged.moveY(dy);
		}
		else
		{
			int dx = e.getX() - press_x;
			int dy = e.getY() - press_y;
			int scroll_x = getScrollPane().getHorizontalScrollBar().getValue();
			int scroll_y = getScrollPane().getVerticalScrollBar().getValue();
			getScrollPane().getHorizontalScrollBar().setValue(scroll_x-dx);
			getScrollPane().getVerticalScrollBar().setValue(scroll_y-dy);
		}
		repaint();
	}

	@Override
	public void mouseMoved(MouseEvent e) {
		nodeDragged = nodeAtPos(e.getX(), e.getY());
		if (nodeDragged != null)
		{
			status.fire("Node"+nodeDragged.getNodeID());
		}
		//repaint();
	}
	
	//===========================================================================================================================================================
	// Component Handler
	//===========================================================================================================================================================	
		
	
	@Override
	public void componentHidden(ComponentEvent e) {
		// Auto-generated method stub
		
	}

	@Override
	public void componentMoved(ComponentEvent e) {
		// Auto-generated method stub
		
	}

	@Override
	public void componentResized(ComponentEvent e) {
		// Auto-generated method stub
		
	}

	@Override
	public void componentShown(ComponentEvent e) {
		repaint();		
	}
	
	//===========================================================================================================================================================
	// Mouse Event Handler Functions
	//===========================================================================================================================================================	
	
	private int counter = 2;
	
	private void addEmptyNode(MouseEvent e) {
		/* Logic domain */
		GuiWSNNode node = new GuiWSNNode(counter++, e.getX(), e.getY());
		nodeList.add(node);
	}
	
	private void delNode(GuiWSNNode node) {
		/* Logic domain */
		if (node != null)
		{
			nodeList.remove(node);
		}
	}
	
	public JScrollPane getScrollPane(){
		return scrollPane;
	}
}
