package gdefalll.gui;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Point2D;
import java.awt.RenderingHints;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;

/**
 * GuiWSNNode class. It will be used in extended version with simulator.
 * License: BUT OPEN SOURCE LICENCE v.1
 * 
 * @author Jan Horacek <ihoracek@fit.vutbr.cz>
 */
public class GuiWSNNode extends GuiDragable {
	
	private static final long serialVersionUID = 0L;
	
	private Color CNode = Color.getHSBColor(0.19f, 0.42f, 0.80f); 
	private static Color CNodeNumber = Color.black;
	private static Integer Radius = 20;
	private static Font FNode = new Font(Font.DIALOG, Font.PLAIN, 14);
	private Integer _node_id = 0;
	
	/**
	 * Constructor of the GUI Node objects.
	 * @param node
	 */
	public GuiWSNNode(int node, int pos_x, int pos_y){
		_node_id = node;
		setX(pos_x);
		setY(pos_y);
	}
	
	@Override
	public void paintComponent(Graphics g){
		Graphics2D g2 = (Graphics2D)g;
		/* Draw Basic Shape */
		drawShape(g2);
		/* Draw Agent */
		//drawAgent(g2);
		/* Draw Node Number */
		drawNodeNumber(g2);
		/* Draw LEDs */
		//drawLEDs(g2);
	}
	
	private void drawShape(Graphics2D g2){
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setColor(CNode);
		int x = getX() - Radius;
		int y = getY() - Radius;
		g2.fillOval(x, y, 2*Radius, 2*Radius);
	}
	
	private void drawNodeNumber(Graphics2D g2){		
		g2.setFont(FNode);
		g2.setColor(CNodeNumber);
		FontMetrics fm = g2.getFontMetrics();		
		String numberLabel = _node_id.toString();
		float wcorr = fm.stringWidth(numberLabel)/2f;
		float hcorr = fm.getHeight()/2f;		
		g2.drawString(numberLabel, getX() - wcorr, getY() + hcorr -4);		
	}
	
	public boolean isAtPos(int x, int y)
	{
		return (distance(x,y) < Radius);
	}
	
	private int distance(int pos_x, int pos_y){
		Point2D center = new Point2D.Double(getX(), getY());
		Point2D destination = new Point2D.Double(pos_x, pos_y);
		return (int)center.distance(destination);
	}
	
	public int getEast(){
		return getX() + Radius;
	}
	
	public int getWest(){
		return getX() - Radius;
	}
	
	public int getNorth(){
		return getY() - Radius;
	}
	
	public int getSouth(){
		return getY() + Radius;
	}
	
	public Integer getNodeID()
	{
		return _node_id;
	}
}
