package gdefalll.gui;

import java.awt.EventQueue;

import javax.swing.JFrame;
import javax.swing.JTabbedPane;
import java.awt.BorderLayout;
import javax.swing.JPanel;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;
import javax.swing.KeyStroke;
import java.awt.event.KeyEvent;
import java.awt.event.InputEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Dimension;
import javax.swing.JLabel;
import org.eclipse.wb.swing.FocusTraversalOnArray;
import java.awt.Component;
import java.awt.Font;
import gdefalll.gui.dialogs.AboutDialog;
import gdefalll.gui.dialogs.AgentNameEditor;
import gdefalll.events.StatusChangeEvent;
import gdefalll.events.StatusChangeListener;
import gdefalll.events.StringContentListener;

import javax.swing.ImageIcon;
import java.awt.Toolkit;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

/**
 * Main class.
 * License: BUT OPEN SOURCE LICENCE v.1
 * 
 * @author Jan Horacek <ihoracek@fit.vutbr.cz>
 */
public class MainWindow {


	private JFrame frmGdeForAlll;
	private final JPanel StatusBar = new JPanel();
	StatusHandler statusHandler = new StatusHandler("Welcome to the GDE for ALLL");
	private final JTabbedPane tabbedPane = new JTabbedPane(JTabbedPane.TOP);

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					MainWindow window = new MainWindow();
					window.frmGdeForAlll.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the application.
	 */
	public MainWindow() {
		initialize();
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
		frmGdeForAlll = new JFrame();
		frmGdeForAlll.setIconImage(Toolkit.getDefaultToolkit().getImage(MainWindow.class.getResource("/gdefalll/Resources/icons/application_double.png")));
		frmGdeForAlll.setTitle("GDE for ALLL");
		frmGdeForAlll.setBounds(100, 100, 450, 300);
		frmGdeForAlll.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frmGdeForAlll.setExtendedState(frmGdeForAlll.getExtendedState() | JFrame.MAXIMIZED_BOTH);

		tabbedPane.setDoubleBuffered(true);
		tabbedPane.setFont(new Font("Dialog", Font.PLAIN, 12));
		frmGdeForAlll.getContentPane().add(tabbedPane, BorderLayout.CENTER);
		
		//GuiTopology topology = new GuiTopology();
		//tabbedPane.addTab("Topology", new ImageIcon(MainWindow.class.getResource("/gdefalll/Resources/icons/chart_organisation.png")), topology.getScrollPane(), null);
		
		// PROBLEM comment addTab topology for Design View, use topologyV below
//		JPanel topologyV = new JPanel();
//		tabbedPane.addTab("Topology", new ImageIcon(MainWindow.class.getResource("/gdefalll/Resources/icons/chart_organisation.png")), topologyV, null);
		
		//JPanel Debug = new JPanel();
		//tabbedPane.addTab("Debug", new ImageIcon(MainWindow.class.getResource("/gdefalll/Resources/icons/bug.png")), Debug, null);
		
		
		GuiAgentEditor agent_editor = new GuiAgentEditor();
		agent_editor.status.addListener(statusHandler);
		//tabbedPane.addTab("New Agent", new ImageIcon(MainWindow.class.getResource("/gdefalll/Resources/icons/user.png")), agent_editor, null);
		
		StatusBar.setSize(new Dimension(0, 20));
		frmGdeForAlll.getContentPane().add(StatusBar, BorderLayout.SOUTH);
		StatusBar.setLayout(new BorderLayout(0, 0));
		JSeparator separator_2 = new JSeparator();
		StatusBar.add(separator_2, BorderLayout.NORTH);
		
		statusHandler.setFont(new Font("Dialog", Font.PLAIN, 12));
		StatusBar.add(statusHandler, BorderLayout.EAST);
		StatusBar.setFocusTraversalPolicy(new FocusTraversalOnArray(new Component[]{statusHandler}));
		//topology.status.addListener(this);
		
		JMenuBar menuBar = new JMenuBar();
		frmGdeForAlll.setJMenuBar(menuBar);
		
		JMenu mnFile = new JMenu("File");
		mnFile.setFont(new Font("Dialog", Font.PLAIN, 12));
		menuBar.add(mnFile);
		
		JMenuItem mNew = new JMenuItem("New");
		mNew.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				addNewAgent();
			}
		});
		mNew.setIcon(new ImageIcon(MainWindow.class.getResource("/gdefalll/Resources/icons/page.png")));
		mNew.setFont(new Font("Dialog", Font.BOLD, 12));
		mNew.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
		mnFile.add(mNew);
		
		JSeparator separator_1 = new JSeparator();
		mnFile.add(separator_1);
		
		JMenuItem mLoad = new JMenuItem("Load");
		mLoad.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				loadAgent();
			}
		});
		mLoad.setIcon(new ImageIcon(MainWindow.class.getResource("/gdefalll/Resources/icons/page_white_edit.png")));
		mLoad.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, InputEvent.CTRL_MASK));
		mnFile.add(mLoad);
		
		JMenuItem mSave = new JMenuItem("Save");
		mSave.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				((GuiAgentEditor)tabbedPane.getSelectedComponent()).save();
			}
		});
		mSave.setIcon(new ImageIcon(MainWindow.class.getResource("/gdefalll/Resources/icons/page_save.png")));
		mSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
		mnFile.add(mSave);
		
		JSeparator separator = new JSeparator();
		mnFile.add(separator);
		
		JMenuItem mExit = new JMenuItem("Exit");
		mExit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				System.exit(0);
			}
		});
		mExit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_MASK));
		mnFile.add(mExit);
		
		JMenu mnAbout = new JMenu("About");
		mnAbout.setFont(new Font("Dialog", Font.PLAIN, 12));
		menuBar.add(mnAbout);
		
		JMenuItem mntmAbout = new JMenuItem("About");
		mntmAbout.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				AboutDialog about = new AboutDialog();
				about.setVisible(true);
			}
		});
		mnAbout.add(mntmAbout);
		
		JMenu mnAgents = new JMenu("Agents");
		mnAgents.setFont(new Font("Dialog", Font.PLAIN, 12));
		//menuBar.add(mnAgents);
		
		JMenuItem mNewAgent = new JMenuItem("New");
		mNewAgent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				
			}
		});
		mNewAgent.setIcon(new ImageIcon(MainWindow.class.getResource("/gdefalll/Resources/icons/user_add.png")));
		mNewAgent.setFont(new Font("Dialog", Font.BOLD, 12));
		mnAgents.add(mNewAgent);
		
		JMenuItem mImportAgent = new JMenuItem("Import");
		mImportAgent.setIcon(new ImageIcon(MainWindow.class.getResource("/gdefalll/Resources/icons/user_go.png")));
		mImportAgent.setFont(new Font("Dialog", Font.BOLD, 12));
		mnAgents.add(mImportAgent);
	}
	
	/**
	 * @author Jan Horacek <ihoracek@fit.vutbr.cz>
	 * Class to implement label that can listen on statusReceived event. It is displayed in status bar.
	 */
	private class StatusHandler extends JLabel implements StatusChangeListener
	{
		private static final long serialVersionUID = 1L;

		private StatusHandler(String text)
		{
			super(text);
		}
		
		public void statusReceived( StatusChangeEvent event )
		{
			setText(event.statusmessage().toString());
		}
	}
	
	/**
	 * Loads an agent from a file. 
	 * @return Agent tab panel.
	 */
	private GuiAgentEditor loadAgent()
	{
		GuiAgentEditor agent_editor = new GuiAgentEditor();
		agent_editor.open();
		return createAgentTab(agent_editor);
	}
	
	/**
	 * Create new agent as GuiAgentEditor in a new tab. 
	 * @return Agent tab panel.
	 */
	private GuiAgentEditor addNewAgent()
	{
		GuiAgentEditor agent_editor = new GuiAgentEditor();
		return createAgentTab(agent_editor);
	}
	
	/**
	 * Creates tab with GuiAgentEditor component. It creates also the close button at each plan tab.
	 * @param agent_editor
	 * @return Agent tab panel.
	 */
	private GuiAgentEditor createAgentTab(GuiAgentEditor agent_editor)
	{
		agent_editor.status.addListener(statusHandler);
		tabbedPane.addTab(agent_editor.getName(), new ImageIcon(MainWindow.class.getResource("/gdefalll/Resources/icons/user.png")), agent_editor, null);
		JLabel tabCloseButton = new JLabel();			
		tabCloseButton.setIcon(MyIcons.getIcon(MyIcons.Type.CLOSE_TAB_BTN));
		tabCloseButton.setSize(10, 10);
		int actualIndex = tabbedPane.getTabCount() - 1;
		
		/**
		 * @author Jan Horacek <ihoracek@fit.vutbr.cz>
		 * Class to handle event after clicking on close button in tab header.
		 */
		class CloseAdapter extends MouseAdapter {
			GuiAgentEditor component;				
			public CloseAdapter(GuiAgentEditor component)
			{
				this.component = component;
			}
			@Override
			public void mouseClicked(MouseEvent e) {
				tabbedPane.remove(component);
			}
		}
		tabCloseButton.addMouseListener(new CloseAdapter(agent_editor));
		
		/**
		 * @author Jan Horacek <ihoracek@fit.vutbr.cz>
		 * Class to handle event to rename agent.
		 */
		class EditAgentNameAdapter extends MouseAdapter implements StringContentListener {
			GuiAgentEditor component;	
			JLabel componentLabel;
			public EditAgentNameAdapter(GuiAgentEditor component, JLabel componentLabel)
			{
				this.component = component;
				this.componentLabel = componentLabel;
			}
			@Override
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					AgentNameEditor nameEditor = new AgentNameEditor();
					nameEditor.setText(component.getAgentName());
					nameEditor.sg.addListener(this);
					nameEditor.setVisible(true);
				}
			}
			@Override
			public void stringContentReceived(String text) {
				component.setAgentName(text);
				componentLabel.setText(text);
			}
		}
		JLabel tabLabel = new JLabel("New agent");
		tabLabel.addMouseListener(new EditAgentNameAdapter(agent_editor, tabLabel));
		JPanel pnl = new JPanel();
	    pnl.setOpaque(false);
	    pnl.add(tabLabel);
	    pnl.add(tabCloseButton);
	    tabbedPane.setTabComponentAt(actualIndex, pnl);
	    tabbedPane.setSelectedIndex(actualIndex);
	    return agent_editor;
	}
}
