package gdefalll.gui;

import javax.swing.ImageIcon;

/**
 * Class that contains icons for actions and buttons.
 * Singleton class! There is accessible enum Type and there should be called getIcon(Type icon) static function only.
 * It was necessary to get an instance of class to be able to load icon from resources.
 * License: BUT OPEN SOURCE LICENCE v.1
 * 
 * @author Jan Horacek <ihoracek@fit.vutbr.cz>
 */
public class MyIcons {
	/**
	 * Singleton instance.
	 */
	private static MyIcons instance;

	private MyIcons() {
	}

	/**
	 * Hidden from outside classes. Used with getIcon(Type icon) static function only.
	 * @return Singleton instance of this object.
	 */
	private synchronized static MyIcons getInstance() {
		if (instance == null)
			instance = new MyIcons();
		return instance;
	}

	/**
	 * Type of icons that we know.
	 * @author Jan Horacek <ihoracek@fit.vutbr.cz>
	 */
	public enum Type {
		ADD_BB, DEL_BB, CHECK_BB, CHECK_IB, WAIT_TIME, WAIT_MSG,
		WAIT_MSG_START, LEDS, MESSAGE, SUBPLAN, REMOTE_PLAN, CATCH_SECTION,
		NEW_ACTION, CLOSE_TAB_BTN
	};

	/**
	 * @param icon
	 * @return Icon of desired type.
	 * @see MyIcons.Type
	 */
	public static ImageIcon getIcon(Type icon) {
		switch (icon) {
		case ADD_BB:
			return new ImageIcon(MyIcons.getInstance().getClass()
					.getResource("/gdefalll/Resources/icons/add.png"));
		case DEL_BB:
			return new ImageIcon(MyIcons.getInstance().getClass()
					.getResource("/gdefalll/Resources/icons/delete.png"));
		case CHECK_BB:
			return new ImageIcon(MyIcons.getInstance().getClass()
					.getResource("/gdefalll/Resources/icons/zoom.png"));
		case CHECK_IB:
			return new ImageIcon(MyIcons.getInstance().getClass()
					.getResource("/gdefalll/Resources/icons/email_open.png"));
		case WAIT_TIME:
			return new ImageIcon(MyIcons.getInstance().getClass()
					.getResource("/gdefalll/Resources/icons/time.png"));
		case WAIT_MSG:
			return new ImageIcon(MyIcons.getInstance().getClass()
					.getResource("/gdefalll/Resources/icons/time_go.png"));
		case WAIT_MSG_START:
			return new ImageIcon(MyIcons.getInstance().getClass()
					.getResource("/gdefalll/Resources/icons/timeline_marker.png"));
		case LEDS:
			return new ImageIcon(MyIcons.getInstance().getClass()
					.getResource("/gdefalll/Resources/icons/lightbulb.png"));
		case MESSAGE:
			return new ImageIcon(MyIcons.getInstance().getClass()
					.getResource("/gdefalll/Resources/icons/email_go.png"));
		case SUBPLAN:
			return new ImageIcon(MyIcons.getInstance().getClass()
					.getResource("/gdefalll/Resources/icons/folder_page.png"));
		case REMOTE_PLAN:
			return new ImageIcon(MyIcons.getInstance().getClass()
					.getResource("/gdefalll/Resources/icons/folder_link.png"));
		case CATCH_SECTION:
			return new ImageIcon(MyIcons.getInstance().getClass()
					.getResource("/gdefalll/Resources/icons/exclamation.png"));
		case NEW_ACTION:
			return new ImageIcon(MyIcons.getInstance().getClass()
					.getResource("/gdefalll/Resources/icons/pencil_add.png"));
		case CLOSE_TAB_BTN:
			return new ImageIcon(MyIcons.getInstance().getClass()
					.getResource("/gdefalll/Resources/icons/cancel.png"));
		}
		return null;
	}
}
