package gdefalll.gui.actions;

import java.awt.Color;
import java.awt.Point;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.SwingUtilities;

import gdefalll.events.StringContentListener;
import gdefalll.gui.MyIcons;
import gdefalll.gui.dialogs.LedsEditor;

/**
 * Leds output action.
 * License: BUT OPEN SOURCE LICENCE v.1
 * 
 * @author Jan Horacek <ihoracek@fit.vutbr.cz>
 */
public class Leds extends ALLLAction implements StringContentListener{
	private static final long serialVersionUID = 1L;
	/**
	 * Label to show output.
	 */
	JLabel value_label = new JLabel();
	/**
	 * value of action. Contains "RED/GREEN/YELLOW ON/OFF/TOGGLE" or register identifier.
	 */
	String value = "RED ON";

	public Leds() {
		super();		
		value_label.setBackground(this.getBackground());
		value_label.setBounds(getWidth()-130, 0, 130, getHeight());
		value_label.setText(value);
		add(value_label);
	}

	@Override
	ImageIcon getIcon() {		
		return MyIcons.getIcon(MyIcons.Type.LEDS);
	}

	@Override
	public String getIdentifier() {
		return "l";
	}

	@Override
	String getLabelText() {
		return "Led output";
	}

	@Override
	Color getLabelColor() {
		return new Color(0, 128, 0);
	}

	@Override
	public void showEditor() {
		Point position = new Point(getLocation()); 
		SwingUtilities.convertPointToScreen(position, this);
		LedsEditor editor = new LedsEditor();
		position.x = position.x - editor.getWidth() / 2 + this.getWidth() / 2;
		position.y = position.y - editor.getHeight() / 2 + this.getHeight() / 2;
		editor.setLocation(position);
		editor.setText(value);
		editor.sg.addListener(this);
		editor.setVisible(true);
	}

	@Override
	public String getALLLCode() {
		return "$(" + getIdentifier() + ",(" + convertLabeltoALLL(value) + "))";
	}
	
	/**
	 * Convert internal value representation to ALLL code.
	 * @param value
	 * @return ALLL code
	 * @see value
	 */
	private String convertLabeltoALLL(String value)
	{
		if (value.equals("RED ON"))
		{
			return "r,1";
		}
		else if (value.equals("RED OFF"))
		{
			return "r,0";
		}
		else if (value.equals("GREEN ON"))
		{
			return "g,1";
		}
		else if (value.equals("GREEN OFF"))
		{
			return "g,0";
		}
		else if (value.equals("YELLOW ON"))
		{
			return "y,1";
		}
		else if (value.equals("YELLOW OFF"))
		{
			return "y,0";
		}
		else if (value.equals("RED TOGGLE"))
		{
			return "r";
		}
		else if (value.equals("GREEN TOGGLE"))
		{
			return "g";
		}
		else if (value.equals("YELLOW TOGGLE"))
		{
			return "y";
		}
		else if (value.equals("&1"))
		{
			return "&1";
		}
		else if (value.equals("&2"))
		{
			return "&2";
		}
		else if (value.equals("&3"))
		{
			return "&3";
		}
		return null;
	}

	@Override
	public void stringContentReceived(String text) {
		value = text;
		if (value.equals("&1"))
    	{
			value_label.setForeground(new Color(153, 153, 255));
			value_label.setText("Reg &1 value");
    	}
    	else if (value.equals("&2"))
    	{
    		value_label.setForeground(new Color(153, 153, 255));
    		value_label.setText("Reg &2 value");
    	}
    	else if (value.equals("&3"))
    	{
    		value_label.setForeground(new Color(153, 153, 255));
    		value_label.setText("Reg &3 value");
    	}
    	else
    	{
    		value_label.setForeground(new Color(0, 0, 0));
    		value_label.setText(text);
    	}
		this.ACG.fireActionChanged();
	}

	@Override
	public void setParameters(String params) {
		stringContentReceived(params);
	}
}
