package gdefalll.gui.actions;

import java.awt.Color;
import java.awt.Point;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.SwingUtilities;
import gdefalll.events.PlanChangedListener;
import gdefalll.events.StringContentListener;
import gdefalll.gui.GuiPlan;
import gdefalll.gui.MyIcons;
import gdefalll.gui.dialogs.RemotePlanEditor;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Vector;

/**
 * Remote plan action.
 * License: BUT OPEN SOURCE LICENCE v.1
 * 
 * @author Jan Horacek <ihoracek@fit.vutbr.cz>
 */
public class RemotePlan extends ALLLAction implements StringContentListener, PlanChangedListener {
	private static final long serialVersionUID = 1L;
	/**
	 * Label to show value.
	 */
	JLabel value_label = new JLabel();
	/**
	 * Text name of reference to external plan.
	 */
	String value = "null";
	/**
	 * The plan list to check if value link is valid
	 */
	private ArrayList<GuiPlan> planList;
	/**
	 * Converted planList to vector. Has to be created and updated with getAvailablePlans().
	 * Used to fill combo box options.
	 * @see getAvailablePlans
	 */
	Vector<String> plans;

	/**
	 * @param planList Can be given later using setPlanList().
	 * @see setPlanList
	 */
	public RemotePlan(ArrayList<GuiPlan> planList) {
		super();		
		this.planList = planList;
		plans = getAvailablePlans();
		if (!plans.isEmpty())
		{
			value = plans.get(0);
		}		
		value_label.setBackground(this.getBackground());		
		value_label.setBounds(getWidth()-130, 0, 130, getHeight());
		value_label.setText(value);
		checkValueValid();
		add(value_label);
	}
	
	public RemotePlan() {
		super();
		plans = getAvailablePlans();
		if (!plans.isEmpty())
		{
			value = plans.get(0);
		}		
		value_label.setBackground(this.getBackground());		
		value_label.setBounds(getWidth()-130, 0, 130, getHeight());
		value_label.setText(value);
		checkValueValid();
		add(value_label);
	}
	
	/**
	 * Set planList that is necessary for checking reference validity.
	 * @param planList
	 */
	public void setPlanList(ArrayList<GuiPlan> planList)
	{
		this.planList = planList;
		plans = getAvailablePlans();
	}

	@Override
	ImageIcon getIcon() {		
		return MyIcons.getIcon(MyIcons.Type.REMOTE_PLAN);
	}

	@Override
	public String getIdentifier() {
		return "^";
	}

	@Override
	String getLabelText() {
		return "Call plan";
	}

	@Override
	Color getLabelColor() {
		return new Color(0, 128, 0);
	}

	@Override
	public void showEditor() {
		Point position = new Point(getLocation()); 
		SwingUtilities.convertPointToScreen(position, this);
		RemotePlanEditor editor = new RemotePlanEditor(plans);
		position.x = position.x - editor.getWidth() / 2 + this.getWidth() / 2;
		position.y = position.y - editor.getHeight() / 2 + this.getHeight() / 2;
		editor.setLocation(position);
		editor.sg.addListener(this);
		editor.setVisible(true);
	}

	@Override
	public String getALLLCode() {
		return getIdentifier() + "(" + value_label.getText() + ")";
	}
	
	/**
	 * Convert planList property to Vector.
	 * @return Vector of names of plans.
	 */
	private Vector<String> getAvailablePlans()
	{
		Vector<String> output = new Vector<String>();
		if (planList != null)
		{
			Iterator<GuiPlan> pi = planList.iterator();
			if (pi.hasNext()) {
				pi.next();
			}
			while (pi.hasNext()) {
				GuiPlan plan = pi.next();
				output.add(plan.getName());
			}
		}		
		return output;
	}
	
	/**
	 * Check if value represent one of plan name that our agent has in Plan Base.
	 * @see value
	 * @see getAvailablePlans
	 */
	private void checkValueValid()
	{
		if (plans.contains(value))
		{
			value_label.setForeground(Color.BLACK);
		}
		else
		{
			value_label.setForeground(Color.RED);
		}
	}

	@Override
	public void stringContentReceived(String text) {
		value = text;
		value_label.setText(value);
		this.ACG.fireActionChanged();
	}

	@Override
	public void planChanged() {
		plans = getAvailablePlans();
		checkValueValid();
	}

	@Override
	public void setParameters(String params) {
		stringContentReceived(params);
	}
}
