package gdefalll.gui.actions;

import gdefalll.events.StringContentListener;
import gdefalll.gui.MyIcons;
import gdefalll.gui.dialogs.WaitTimeEditor;
import java.awt.Color;
import java.awt.Point;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.SwingUtilities;

/**
 * Wait for specific time action.
 * License: BUT OPEN SOURCE LICENCE v.1
 * 
 * @author Jan Horacek <ihoracek@fit.vutbr.cz>
 */
public class WaitTime extends ALLLAction implements StringContentListener{
	
	private static final long serialVersionUID = 1L;
	/**
	 * Label to show value of action.
	 */
	JLabel value_label = new JLabel();
	/**
	 * Current value of action. Nuber/register identifier.
	 */
	String value = "1000"; 

	public WaitTime() {
		super();		
		value_label.setBackground(this.getBackground());
		value_label.setBounds(getWidth()-130, 0, 130, getHeight());
		value_label.setText(value + " ms");
		add(value_label);
	}
	
	/**
	 * Show value of this action.
	 * @see value
	 */
	private void showOutputValue()
	{
		try
		{ 
			int output = Integer.parseInt(value);
			value_label.setForeground(new Color(0, 0, 0));
			value_label.setText(output + " ms");
	    } catch( NumberFormatException e )
	    { 
	    	value_label.setForeground(new Color(153, 153, 255));
			value_label.setText("Reg " + value + " value");
	    }		
	}

	@Override
	ImageIcon getIcon() {		
		return MyIcons.getIcon(MyIcons.Type.WAIT_TIME);
	}

	@Override
	public String getIdentifier() {
		return "w";
	}

	@Override
	String getLabelText() {
		return "Wait";
	}

	@Override
	Color getLabelColor() {
		return new Color(0, 128, 0);
	}

	@Override
	public void showEditor() {
		Point position = new Point(getLocation()); 
		SwingUtilities.convertPointToScreen(position, this);
		WaitTimeEditor editor = new WaitTimeEditor();
		position.x = position.x - editor.getWidth() / 2 + this.getWidth() / 2;
		position.y = position.y - editor.getHeight() / 2 + this.getHeight() / 2;
		editor.setLocation(position);
		editor.setText(value);
		editor.sg.addListener(this);
		editor.setVisible(true);
	}

	@Override
	public String getALLLCode() {
		try
		{ 
			Integer.parseInt(value);
			return "$(" + getIdentifier() + ",(" + value + "))";
	    } catch( NumberFormatException e )
	    {
	    	return "$(" + getIdentifier() + "," + value + ")";
	    }	
	}

	@Override
	public void stringContentReceived(String text) {
		value = text;
		showOutputValue();
		this.ACG.fireActionChanged();
	}

	@Override
	public void setParameters(String params) {
		stringContentReceived(params);
	}

}
